#!/usr/bin/perl
#
# apache-stats
#
# A utility for use with cricket to pull some interesting information from 
# an Apache web server (http://www.apache.org) and graph it.
#
# Copyright (C) 2000  Andrew Williams
# v1.0 modifications by Daniel V. Klein, Copyright 2001
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Output is as follows (use the line #'s in the exec calls from cricket)
#
# 0-8:  are pulled directly from the output of the mod_status output
# 9-16: are derived from the scoreboard output.  They are counts of
#       processes in the named state.
# 17-18:are derived from the current and previous output of mod_status
# 
# 0: Total hits
# 1: Total kB served
# 2: CPU Load (in %)
# 3: Uptime (in seconds)
# 4: Requests per Second (average since server reboot)
# 5: Bytes per Second (average since server reboot)
# 6: Busy Servers
# 7: Idle Servers
# 8: Scoreboard
# 9: Waiting
# 10: Starting
# 11: Reading
# 12: Writing
# 13: Keepalive
# 14: DNS Lookup
# 15: Logging
# 16: Gracefully Finishing
# 17: Requests per second (since last check)
# 18: Bytes per second (since last check)

use LWP::UserAgent;
use FileHandle;

die("Usage: $0 <URL> <delta-file>\n")	unless @ARGV == 2;

# Set the URL to the server status page
($URL, $DELTA) = @ARGV;

unless ($URL =~ /^http:/i) {
    die("Must provide a full url you sent: $URL\n");
    }
unless (sysopen DELTA, $DELTA, O_RDWR|O_CREAT) {
    die("Cannot open $DELTA - $!\n");
    }
chomp($old_Uptime = <DELTA>);
chomp($old_Accesses = <DELTA>);
chomp($old_kBytes = <DELTA>);
seek DELTA, 0, 0;

# How long to wait for a reply
$timeout = 30;


## BEGIN ##

# Do some basic setup of the UserAgent package
$ua = new LWP::UserAgent;
$ua->agent("apache-stats/1.1" . $ua->agent);
$ua->timeout($timeout);


# Setup the request ?auto is appended to the URL to get the result in a 
# more machine readable format

$request = new HTTP::Request('GET', $URL . "?auto");
$response = $ua->request($request);

#
# Collect the results into a hash table, and calculate the true hits/sec and
# kbytes/sec.  Write the latest values back out to the delta file.
#
%ans = split/:\s*|\n/, $response->content;
warn "unexpected (non-server-status) data seen"	unless keys %ans == 10;
if ($old_Uptime >= $ans{Uptime}) {
    $true_Accesses_per_sec = ${"Total Accesses"} / $ans{Uptime};
    $true_Bytes_per_sec = ${"Total kBytes"}*1024 / $ans{Uptime};
    }
else {
    $delta = $ans{Uptime} - $old_Uptime;
    $true_Accesses_per_sec = ($ans{"Total Accesses"} - $old_Accesses) / $delta;
    $true_Bytes_per_sec = ($ans{"Total kBytes"} - $old_kBytes)*1024 / $delta;
    }
print DELTA qq/$ans{Uptime}\n$ans{"Total Accesses"}\n$ans{"Total kBytes"}\n/;
close DELTA;


#
# Process the scoreboard file counts - tr/// returns number of chars "changed"
# _ waiting for connection
# S Starting up
# R Reading Request
# W Sending Reply
# K Keepalive (read)
# D DNS Lookup
# L Logging
# G Gracefully finishing
# . Open slot (not counted)
#

$Waiting    = $ans{Scoreboard} =~ tr/_/_/;
$Starting   = $ans{Scoreboard} =~ tr/S/S/;
$Reading    = $ans{Scoreboard} =~ tr/R/R/;
$Writing    = $ans{Scoreboard} =~ tr/W/W/;
$Keepalive  = $ans{Scoreboard} =~ tr/K/K/;
$DNS	    = $ans{Scoreboard} =~ tr/D/D/;
$Logging    = $ans{Scoreboard} =~ tr/L/L/;
$Graceful   = $ans{Scoreboard} =~ tr/G/G/;

print <<"==END==";
$ans{"Total Accesses"}
$ans{"Total kBytes"}
$ans{CPULoad}
$ans{Uptime}
$ans{ReqPerSec}
$ans{BytesPerSec}
$ans{BytesPerReq}
$ans{BusyServers}
$ans{IdleServers}
$Waiting
$Starting
$Reading
$Writing
$Keepalive
$DNS
$Logging
$Graceful
$true_Accesses_per_sec
$true_Bytes_per_sec
==END==
